@@ -1,6 +1,6 @@
-import System.IO
+{-# OPTIONS_GHC -fno-warn-missing-signatures #-}
+
 import Network.HaskellNet.IMAP
-import qualified Data.ByteString.Char8 as BS
 import Control.Monad
 
 imapServer = "imap.mail.org"
@@ -11,8 +11,8 @@ main = do
   con <- connectIMAP imapServer
   login con username password
   mboxes <- list con
-  mapM print mboxes
+  mapM_ print mboxes
   select con "INBOX"
   msgs <- search con [ALLs]
-  mapM_ (\x -> print x) (take 4 msgs)
-  forM_ (take 4msgs) (\x -> fetch con x >>= print)
\ No newline at end of file
+  mapM_ print (take 4 msgs)
+  forM_ (take 4msgs) (fetch con >=> print)
